/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the Computer Systems
 *       Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ------------
 *
 * Filename: mplug-unix.h
 *   -- Created on Sat Dec 27 1997
 *   -- Author: Yatin Chawathe <yatin@>
 *
 * Description:
 *
 *
 *  @(#) $Header: /usr/src/mash/repository/mash/mash-1/mplug/unix/mplug-unix.h,v 1.1 1997/12/28 06:26:07 yatin Exp $
 */

#ifndef MASH_MPLUG_UNIX_H
#define MASH_MPLUG_UNIX_H

#include "../mplug.h"
#include <X11/Intrinsic.h>

// there is a bug in netscape (at least under Unix) where it doesn't seem
// to call the plugin's NPP_SetWindow procedure when the main window is
// resized
#define NETSCAPE_RESIZE_BUG 1



// define platform-specific features in a sub-class

class UnixMPlugInstance : public MPlugInstance {
public:
	UnixMPlugInstance(NPMIMEType mimeType, NPP instance, uint16 mode,
			  int16 argc, const char *const *argn,
			  const char *const *argv);
	~UnixMPlugInstance();

	NPError NewWindow(NPWindow *npWindow);
	NPError ReconfigureWindow(NPWindow *npWindow);
	NPError DeleteWindow();

	const char *GetDisplay(NPWindow *npWindow);
	unsigned long GetWindowHandle() { return (unsigned long) window_; }

private:
	Window window_;
	
#ifdef NETSCAPE_RESIZE_BUG
	Widget parentWidget_;
	static void StructureNotifyHandler(Widget w, XtPointer closure,
					   XEvent *event, Boolean *dummy);
#endif
};


#endif /* #ifdef MPLUG_UNIX_H */ 
