import Prefops/Binary

#
# Prefops for Stockcast appplication.
# Receiver reports only include those symbols interested.
# lpv: {sym1 sym2 ... symn}
# pv: {sym [0|1]}
#
Class Prefops/Stockcast -superclass Prefops/Binary

#
# Find the cost of the current partition.
# It is defined to be the percentage of extra symbols than requested a
# receiver is getting, on average. 
#
Prefops/Stockcast public get_cost { alg } {
	$self instvar pref_

	array set cluts [$alg get_cluts]
	set num_cluts [$alg set num_cluts_]
	set pc 0
	foreach recv_id [array names pref_] {
		set ex 0
		set total 0
		puts "pref($recv_id): $pref_($recv_id)"
		for {set i 1} {$i <= $num_cluts} {incr i} {
			if [info exists cluts($src,$i)] {
				set in [lintersect $pref_($recv_id) $cluts(src,$i)]
				if { $in != "" } {
					set ex [expr $ex + ([llength $cluts(src,$i)] - [llength $in])]
					set total [expr $total + [llength $cluts($src,$i)]]
				}
			}
		}
		set pc [expr double($ex) / double($total)]
	}
	set pc [expr double($pc) / double([llength [array names pref_]])]
	puts stderr "avg percent of extra symbols: $pc"
	return $pc
}

Prefops/Stockcast public acceptable { cost thresh } {
	if { $cost <= $thresh } {
		return 1
	} else {
		return 0
	}
}

#
# Return the percentage of receivers interested in source src_id.
#
Prefops/Stockcast public get_wt { src_id } {
	$self instvar pref_

	set total_cnt 0
	set interest_cnt 0
	foreach recv_id [array names pref_] {
		incr total_cnt
		if { [lsearch $pref_(recv_id) $src_id] != -1 } {
			incr interest_cnt
		}
	}
	return [expr double($interest_cnt) / double($total_cnt)]
}

#
# Return a list of normalized weights receivers puts on source.
#
Prefops/Stockcast public get_wts { } {
	$self instvar sym_list_

	set sum 0
	foreach src_id $sym_list_ {
		set ans($src_id) [$self get_wt $src_id]
		set sum [expr $sum + $ans($src_id)]
	}
	foreach src_id $sym_list_ {
		set ans($src_id) [expr double($ans($src_id)) / double($sum)]
	}
	return [array get ans]
}

#
# Receiver or source (host) has died.
# Aged out the associated records to avoid considering them in alg.
#
Prefops/Stockcast public delete { ctype id } {
	$self instvar alg_ pref_ dataflow_ sym_list_

	# remove receiver from pref database
	# re-calculate the list of symbols interested
	if { "$ctype" == "recv" } {
		unset pref_($id)
		set sym_list ""
		foreach recv_id [array names pref_] {
			set sym_list [lunion $sym_list $pref_($recv_id)]
		}
		set sym_list_ $sym_list

        # remove symbols provided by src from consideration in alg
	} elseif { "$ctype" == "src" } {
		foreach sym $dataflow_($id) {
			set out 1
			foreach src_id [array names dataflow_] {
				if { [lsearch dataflow_($src_id) $sym] != 1 } {
					set out 0
					break
				}
			}
			if { $out == 1 } {
				set sym_list_ [lremove $sym_list_ $sym]
				$alg_ delete_src $sym
			}
		}
		unset dataflow_($id)
	}
}

#
# Heartbeat message from source.
# Keep track of symbols provided by this source.
# lpv: {sym1 sym2 ... symn}
#
Prefops/Stockcast public set_df { src_id lpv } {
	$self instvar dataflow_

	set dataflow_($src_id) $lpv

	# insert into partition trigger by receiver pref report
}

#
# Insert preference vector list lpv of receiver recv_id into database.
# lpv: {sym1 sym2 ... symn}
#
Prefops/Stockcast public set_pv { recv_id lpv } {
	$self instvar alg_ pref_ sym_list_

	# reconfigure only if pref has changed since last report
	if { [info exists pref_(recv_id)] && $pref_(recv_id) == $lpv } {
		return
	}
		
	# first report ever
	if ![info exists sym_list_] {
		set new $lpv
		set sym_list_ $lpv
		
	} else {
		set new [ldiff $lpv $sym_list_]
		set sym_list_ [lunion $lpv $sym_list_]

		# check whether others are still interested in deleted symbols
		if [info exists pref_(recv_id)] {
			set deleted [ldiff $pref_($recv_id) $lpv]
			foreach sym $deleted {
				set out 1
				foreach id [array names pref_] {
					if { "$id" != "$recv_id" && [lsearch $pref_($id) $sym] != -1 } {
						set out 0
						break
					}
				}
				if { $out == 1 } {
					set sym_list_ [lremove $sym_list_ $sym]
					$alg_ delete_src $sym
				}
			}
	        }
	}

	# insert new src into alg
	foreach sym $new {
		$alg_ insert_src $sym
	}

	set pref_($recv_id) $lpv		
}

#
# Find preference vector of receiver recv_id on source sym
# pv: {sym [0|1]}
#
Prefops/Stockcast public get_pv { recv_id sym } {
	$self instvar pref_

	if { [lsearch $pref_($recv_id) $sym] == -1 } {
		return [list $sym 0]
	} else {
		return [list $sym 1]
	}
}

#
# Return all the sources 
#
Prefops/Stockcast public get_srclist { } {
	$self instvar sym_list_
	
	return $sym_list_
}

#
# Return all the receivers
#
Prefops/Stockcast public get_recvlist { } {
	$self instvar pref_
	
	return [array names pref_]
}
